__version__   = '21.2.1'
__author__    = 'Jean-Benoit Aubin <jeaubin@cisco.com>'
__date__      = ''
__contact__   = 'pyats-support@cisco.com'
__copyright__ = 'Cisco System, Inc. Cisco Confedential'

__import__('pkg_resources').declare_namespace(__name__)
